"use strict";
/**
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAbstractClassDeclaration = exports.toFactoryMetadata = exports.wrapTypeReference = exports.wrapFunctionExpressionsInParens = exports.tryUnwrapForwardRef = exports.unwrapExpression = exports.getAngularDecorators = exports.isAngularDecorator = exports.findAngularDecorator = exports.isAngularCore = exports.valueReferenceToExpression = exports.CORE_MODULE = void 0;
const compiler_1 = require("static-injector/transform/compiler");
const typescript_1 = __importDefault(require("typescript"));
/** Module name of the framework core. */
exports.CORE_MODULE = 'static-injector';
function valueReferenceToExpression(valueRef) {
    if (valueRef.kind === 2 /* TypeValueReferenceKind.UNAVAILABLE */) {
        return null;
    }
    else if (valueRef.kind === 0 /* TypeValueReferenceKind.LOCAL */) {
        const expr = new compiler_1.WrappedNodeExpr(valueRef.expression);
        return expr;
    }
    else {
        let importExpr = new compiler_1.ExternalExpr({
            moduleName: valueRef.moduleName,
            name: valueRef.importedName,
        });
        if (valueRef.nestedPath !== null) {
            for (const property of valueRef.nestedPath) {
                importExpr = new compiler_1.ReadPropExpr(importExpr, property);
            }
        }
        return importExpr;
    }
}
exports.valueReferenceToExpression = valueReferenceToExpression;
function isAngularCore(decorator) {
    return decorator.import !== null && decorator.import.from === exports.CORE_MODULE;
}
exports.isAngularCore = isAngularCore;
function findAngularDecorator(decorators, name, isCore) {
    return decorators.find((decorator) => isAngularDecorator(decorator, name, isCore));
}
exports.findAngularDecorator = findAngularDecorator;
function isAngularDecorator(decorator, name, isCore) {
    if (isCore) {
        return decorator.name === name;
    }
    else if (isAngularCore(decorator)) {
        return decorator.import.name === name;
    }
    return false;
}
exports.isAngularDecorator = isAngularDecorator;
function getAngularDecorators(decorators, names, isCore) {
    return decorators.filter((decorator) => {
        const name = isCore ? decorator.name : decorator.import?.name;
        if (name === undefined || !names.includes(name)) {
            return false;
        }
        return isCore || isAngularCore(decorator);
    });
}
exports.getAngularDecorators = getAngularDecorators;
/**
 * Unwrap a `ts.Expression`, removing outer type-casts or parentheses until the expression is in its
 * lowest level form.
 *
 * For example, the expression "(foo as Type)" unwraps to "foo".
 */
function unwrapExpression(node) {
    while (typescript_1.default.isAsExpression(node) || typescript_1.default.isParenthesizedExpression(node)) {
        node = node.expression;
    }
    return node;
}
exports.unwrapExpression = unwrapExpression;
function expandForwardRef(arg) {
    arg = unwrapExpression(arg);
    if (!typescript_1.default.isArrowFunction(arg) && !typescript_1.default.isFunctionExpression(arg)) {
        return null;
    }
    const body = arg.body;
    // Either the body is a ts.Expression directly, or a block with a single return statement.
    if (typescript_1.default.isBlock(body)) {
        // Block body - look for a single return statement.
        if (body.statements.length !== 1) {
            return null;
        }
        const stmt = body.statements[0];
        if (!typescript_1.default.isReturnStatement(stmt) || stmt.expression === undefined) {
            return null;
        }
        return stmt.expression;
    }
    else {
        // Shorthand body - return as an expression.
        return body;
    }
}
/**
 * If the given `node` is a forwardRef() expression then resolve its inner value, otherwise return
 * `null`.
 *
 * @param node the forwardRef() expression to resolve
 * @param reflector a ReflectionHost
 * @returns the resolved expression, if the original expression was a forwardRef(), or `null`
 *     otherwise.
 */
function tryUnwrapForwardRef(node, reflector) {
    node = unwrapExpression(node);
    if (!typescript_1.default.isCallExpression(node) || node.arguments.length !== 1) {
        return null;
    }
    const fn = typescript_1.default.isPropertyAccessExpression(node.expression)
        ? node.expression.name
        : node.expression;
    if (!typescript_1.default.isIdentifier(fn)) {
        return null;
    }
    const expr = expandForwardRef(node.arguments[0]);
    if (expr === null) {
        return null;
    }
    const imp = reflector.getImportOfIdentifier(fn);
    if (imp === null ||
        imp.from !== 'static-injector' ||
        imp.name !== 'forwardRef') {
        return null;
    }
    return expr;
}
exports.tryUnwrapForwardRef = tryUnwrapForwardRef;
const parensWrapperTransformerFactory = (context) => {
    const visitor = (node) => {
        const visited = typescript_1.default.visitEachChild(node, visitor, context);
        if (typescript_1.default.isArrowFunction(visited) || typescript_1.default.isFunctionExpression(visited)) {
            return typescript_1.default.factory.createParenthesizedExpression(visited);
        }
        return visited;
    };
    return (node) => typescript_1.default.visitEachChild(node, visitor, context);
};
/**
 * Wraps all functions in a given expression in parentheses. This is needed to avoid problems
 * where Tsickle annotations added between analyse and transform phases in Angular may trigger
 * automatic semicolon insertion, e.g. if a function is the expression in a `return` statement.
 * More
 * info can be found in Tsickle source code here:
 * https://github.com/angular/tsickle/blob/d7974262571c8a17d684e5ba07680e1b1993afdd/src/jsdoc_transformer.ts#L1021
 *
 * @param expression Expression where functions should be wrapped in parentheses
 */
function wrapFunctionExpressionsInParens(expression) {
    return typescript_1.default.transform(expression, [parensWrapperTransformerFactory])
        .transformed[0];
}
exports.wrapFunctionExpressionsInParens = wrapFunctionExpressionsInParens;
/**
 * Create an R3Reference for a class.
 *
 * The `value` is the exported declaration of the class from its source file.
 * The `type` is an expression that would be used in the typings (.d.ts) files.
 */
function wrapTypeReference(reflector, clazz) {
    const value = new compiler_1.WrappedNodeExpr(clazz.name);
    const type = value;
    return { value, type };
}
exports.wrapTypeReference = wrapTypeReference;
function toFactoryMetadata(meta, target) {
    return {
        name: meta.name,
        type: meta.type,
        typeArgumentCount: meta.typeArgumentCount,
        deps: meta.deps,
        target,
    };
}
exports.toFactoryMetadata = toFactoryMetadata;
function isAbstractClassDeclaration(clazz) {
    return typescript_1.default.canHaveModifiers(clazz) && clazz.modifiers !== undefined
        ? clazz.modifiers.some((mod) => mod.kind === typescript_1.default.SyntaxKind.AbstractKeyword)
        : false;
}
exports.isAbstractClassDeclaration = isAbstractClassDeclaration;
