"use strict";
/*!
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createIndexSignature = exports.createFunctionDeclaration = exports.createImportDeclaration = exports.createParameterDeclaration = void 0;
const typescript_1 = __importDefault(require("typescript"));
/** Whether the current TypeScript version is after 4.9. */
const IS_AFTER_TS_49 = isAfterVersion(4, 9);
/**
 * Creates a `ts.ParameterDeclaration` declaration.
 *
 * TODO(crisbeto): this is a backwards-compatibility layer for versions of TypeScript less than 4.9.
 * We should remove it once we have dropped support for the older versions.
 */
exports.createParameterDeclaration = IS_AFTER_TS_49
    ? typescript_1.default.factory.createParameterDeclaration
    : (modifiers, dotDotDotToken, name, questionToken, type, initializer) => typescript_1.default.factory.createParameterDeclaration(...splitModifiers(modifiers), dotDotDotToken, name, questionToken, type, initializer);
/**
 * Creates a `ts.ImportDeclaration` declaration.
 *
 * TODO(crisbeto): this is a backwards-compatibility layer for versions of TypeScript less than 4.9.
 * We should remove it once we have dropped support for the older versions.
 */
exports.createImportDeclaration = IS_AFTER_TS_49
    ? typescript_1.default.factory.createImportDeclaration
    : (modifiers, importClause, moduleSpecifier, assertClause) => typescript_1.default.factory.createImportDeclaration(undefined, modifiers, importClause, moduleSpecifier, assertClause);
/**
 * Creates a `ts.FunctionDeclaration` declaration.
 *
 * TODO(crisbeto): this is a backwards-compatibility layer for versions of TypeScript less than 4.9.
 * We should remove it once we have dropped support for the older versions.
 */
exports.createFunctionDeclaration = IS_AFTER_TS_49
    ? typescript_1.default.factory.createFunctionDeclaration
    : (modifiers, asteriskToken, name, typeParameters, parameters, type, body) => typescript_1.default.factory.createFunctionDeclaration(...splitModifiers(modifiers), asteriskToken, name, typeParameters, parameters, type, body);
/**
 * Creates a `ts.IndexSignatureDeclaration` declaration.
 *
 * TODO(crisbeto): this is a backwards-compatibility layer for versions of TypeScript less than 4.9.
 * We should remove it once we have dropped support for the older versions.
 */
exports.createIndexSignature = IS_AFTER_TS_49
    ? typescript_1.default.factory.createIndexSignature
    : (modifiers, parameters, type) => typescript_1.default.factory.createIndexSignature(modifiers, parameters, type);
/**
 * Splits a `ModifierLike` into two arrays: decorators and modifiers. Used for backwards
 * compatibility with TS 4.7 and below where most factory functions require separate `decorators`
 * and `modifiers` arrays.
 */
function splitModifiers(allModifiers) {
    if (!allModifiers) {
        return [undefined, undefined];
    }
    const decorators = [];
    const modifiers = [];
    for (const current of allModifiers) {
        if (typescript_1.default.isDecorator(current)) {
            decorators.push(current);
        }
        else {
            modifiers.push(current);
        }
    }
    return [
        decorators.length ? decorators : undefined,
        modifiers.length ? modifiers : undefined,
    ];
}
/** Checks if the current version of TypeScript is after the specified major/minor versions. */
function isAfterVersion(targetMajor, targetMinor) {
    const [major, minor] = typescript_1.default.versionMajorMinor
        .split('.')
        .map((part) => parseInt(part));
    if (major < targetMajor) {
        return false;
    }
    return major === targetMajor ? minor >= targetMinor : true;
}
