"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InjectableTransformerFactory = exports.createTransformer = void 0;
const typescript_1 = __importDefault(require("typescript"));
const injectable_1 = require("./compiler-cli/src/ngtsc/annotations/src/injectable");
const imports_1 = require("./compiler-cli/src/ngtsc/imports");
const reflection_1 = require("./compiler-cli/src/ngtsc/reflection");
const transform_1 = require("./compiler-cli/src/ngtsc/transform");
const translator_1 = require("./compiler-cli/src/ngtsc/translator");
const node_Iteration_1 = require("./node-Iteration");
const NO_DECORATORS = new Set();
function createTransformer(program, options) {
    let factory = new InjectableTransformerFactory(program, options);
    return factory.getTransform();
}
exports.createTransformer = createTransformer;
class InjectableTransformerFactory {
    program;
    options;
    typeChecker;
    reflectionHost;
    handler;
    constructor(program, options = {}) {
        this.program = program;
        this.options = options;
        this.typeChecker = this.program.getTypeChecker();
        this.reflectionHost = new reflection_1.TypeScriptReflectionHost(this.typeChecker);
        this.handler = new injectable_1.InjectableDecoratorHandler(this.reflectionHost, false, !!this.options.strictCtorDeps, false, transform_1.CompilationMode.FULL);
    }
    getTransform() {
        return (context) => this.transform(context);
    }
    visit(node, context, map) {
        return typescript_1.default.visitEachChild(node, (node) => {
            let result = map.get(node);
            if (result) {
                const filteredDecorators = 
                // Remove the decorator which triggered this compilation, leaving the others alone.
                maybeFilterDecorator(typescript_1.default.getDecorators(node), [
                    result.decorator,
                ]);
                const nodeModifiers = typescript_1.default.getModifiers(node);
                let updatedModifiers;
                if (filteredDecorators?.length || nodeModifiers?.length) {
                    updatedModifiers = [
                        ...(filteredDecorators || []),
                        ...(nodeModifiers || []),
                    ];
                }
                return typescript_1.default.factory.updateClassDeclaration(node, updatedModifiers, node.name, node.typeParameters, node.heritageClauses || [], [
                    ...node.members.map((node) => this._stripAngularDecorators(node)),
                    ...result.members,
                ]);
            }
            return this.visit(node, context, map);
        }, context);
    }
    transform(context) {
        return (sf) => {
            let map = this.preAnalysis(sf);
            sf = this.updateStatements(sf, map.importManager);
            return this.visit(sf, context, map.classMetadataMap);
        };
    }
    preAnalysis(sf) {
        let classMetadataMap = new Map();
        let write = new imports_1.NoopImportRewriter();
        let importManager = new translator_1.ImportManager(write);
        (0, node_Iteration_1.nodeIteration)(sf, (node) => {
            if (typescript_1.default.isClassDeclaration(node) &&
                typescript_1.default.getDecorators(node) &&
                this.reflectionHost.isClass(node)) {
                const decorators = this.reflectionHost.getDecoratorsOfDeclaration(node) || [];
                let result = this.handler.detect(node, decorators);
                if (!result) {
                    return;
                }
                let analysisOutput = this.handler.analyze(node, result.metadata);
                let compileResult = this.handler.compileFull(node, analysisOutput.analysis);
                let resultNode = this.translate(compileResult, importManager);
                classMetadataMap.set(node, {
                    ...resultNode,
                    decorator: result.trigger,
                });
            }
        });
        // this.sourceFileMetadataMap.set(sf, {
        //   classMetadataMap,
        //   importManager: importManager,
        // });
        return { classMetadataMap, importManager };
    }
    translate(compileResult, importManager) {
        // There is at least one field to add.
        const statements = [];
        const members = [];
        for (const field of compileResult) {
            // Translate the initializer for the field into TS nodes.
            const exprNode = (0, translator_1.translateExpression)(field.initializer, importManager);
            // Create a static property declaration for the new field.
            const property = typescript_1.default.factory.createPropertyDeclaration([typescript_1.default.factory.createToken(typescript_1.default.SyntaxKind.StaticKeyword)], field.name, undefined, undefined, exprNode);
            if (false) {
                // Closure compiler transforms the form `Service.ɵprov = X` into `Service$ɵprov = X`. To
                // prevent this transformation, such assignments need to be annotated with @nocollapse.
                // Note that tsickle is typically responsible for adding such annotations, however it
                // doesn't yet handle synthetic fields added during other transformations.
                typescript_1.default.addSyntheticLeadingComment(property, typescript_1.default.SyntaxKind.MultiLineCommentTrivia, '* @nocollapse ', 
                /* hasTrailingNewLine */ false);
            }
            field.statements
                .map((stmt) => (0, translator_1.translateStatement)(stmt, importManager))
                .forEach((stmt) => statements.push(stmt));
            members.push(property);
        }
        return { statements, members };
    }
    /**
     * Remove Angular decorators from a `ts.Node` in a shallow manner.
     *
     * This will remove decorators from class elements (getters, setters, properties, methods) as well
     * as parameters of constructors.
     */
    _stripAngularDecorators(node) {
        const modifiers = typescript_1.default.canHaveModifiers(node)
            ? typescript_1.default.getModifiers(node)
            : undefined;
        const nonCoreDecorators = typescript_1.default.canHaveDecorators(node)
            ? this._nonCoreDecoratorsOnly(node)
            : undefined;
        const combinedModifiers = [
            ...(nonCoreDecorators || []),
            ...(modifiers || []),
        ];
        if (typescript_1.default.isParameter(node)) {
            // Strip decorators from parameters (probably of the constructor).
            node = typescript_1.default.factory.updateParameterDeclaration(node, combinedModifiers, node.dotDotDotToken, node.name, node.questionToken, node.type, node.initializer);
        }
        else if (typescript_1.default.isMethodDeclaration(node)) {
            // Strip decorators of methods.
            node = typescript_1.default.factory.updateMethodDeclaration(node, combinedModifiers, node.asteriskToken, node.name, node.questionToken, node.typeParameters, node.parameters, node.type, node.body);
        }
        else if (typescript_1.default.isPropertyDeclaration(node)) {
            // Strip decorators of properties.
            node = typescript_1.default.factory.updatePropertyDeclaration(node, combinedModifiers, node.name, node.questionToken, node.type, node.initializer);
        }
        else if (typescript_1.default.isGetAccessor(node)) {
            // Strip decorators of getters.
            node = typescript_1.default.factory.updateGetAccessorDeclaration(node, combinedModifiers, node.name, node.parameters, node.type, node.body);
        }
        else if (typescript_1.default.isSetAccessor(node)) {
            // Strip decorators of setters.
            node = typescript_1.default.factory.updateSetAccessorDeclaration(node, combinedModifiers, node.name, node.parameters, node.body);
        }
        else if (typescript_1.default.isConstructorDeclaration(node)) {
            // For constructors, strip decorators of the parameters.
            const parameters = node.parameters.map((param) => this._stripAngularDecorators(param));
            node = typescript_1.default.factory.updateConstructorDeclaration(node, modifiers, parameters, node.body);
        }
        return node;
    }
    /**
     * Return all decorators on a `Declaration` which are from static-injector, or an empty set if none
     * are.
     */
    _angularCoreDecorators(decl) {
        const decorators = this.reflectionHost.getDecoratorsOfDeclaration(decl);
        if (decorators === null) {
            return NO_DECORATORS;
        }
        const coreDecorators = decorators
            .filter((dec) => isFromAngularCore(dec))
            .map((dec) => dec.node);
        if (coreDecorators.length > 0) {
            return new Set(coreDecorators);
        }
        else {
            return NO_DECORATORS;
        }
    }
    /**
     * Given a `ts.Node`, filter the decorators array and return a version containing only non-Angular
     * decorators.
     *
     * If all decorators are removed (or none existed in the first place), this method returns
     * `undefined`.
     */
    _nonCoreDecoratorsOnly(node) {
        const decorators = typescript_1.default.getDecorators(node);
        // Shortcut if the node has no decorators.
        if (decorators === undefined) {
            return undefined;
        }
        // Build a Set of the decorators on this node from @angular/core.
        const coreDecorators = this._angularCoreDecorators(node);
        if (coreDecorators.size === decorators.length) {
            // If all decorators are to be removed, return `undefined`.
            return undefined;
        }
        else if (coreDecorators.size === 0) {
            // If no decorators need to be removed, return the original decorators array.
            return nodeArrayFromDecoratorsArray(decorators);
        }
        // Filter out the core decorators.
        const filtered = decorators.filter((dec) => !coreDecorators.has(dec));
        // If no decorators survive, return `undefined`. This can only happen if a core decorator is
        // repeated on the node.
        if (filtered.length === 0) {
            return undefined;
        }
        // Create a new `NodeArray` with the filtered decorators that sourcemaps back to the original.
        return nodeArrayFromDecoratorsArray(filtered);
    }
    updateStatements(node, importManager) {
        return (0, transform_1.addImports)(typescript_1.default.factory, importManager, node);
    }
}
exports.InjectableTransformerFactory = InjectableTransformerFactory;
/**
 * todo 引入
 */
function isFromAngularCore(decorator) {
    return (decorator.import !== null && decorator.import.from === 'static-injector');
}
/** Creates a `NodeArray` with the correct offsets from an array of decorators. */
function nodeArrayFromDecoratorsArray(decorators) {
    const array = typescript_1.default.factory.createNodeArray(decorators);
    if (array.length > 0) {
        array.pos = decorators[0].pos;
        array.end = decorators[decorators.length - 1].end;
    }
    return array;
}
function maybeFilterDecorator(decorators, toRemove) {
    if (decorators === undefined) {
        return undefined;
    }
    const filtered = decorators.filter((dec) => toRemove.find((decToRemove) => typescript_1.default.getOriginalNode(dec) === decToRemove) === undefined);
    if (filtered.length === 0) {
        return undefined;
    }
    return typescript_1.default.factory.createNodeArray(filtered);
}
